<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0">
  <xsl:output method="xml"/>

    <!--=========================================================================
            input pattern:  P
    ==========================================================================-->
    <xsl:template match="InputPattern[not(ChannelSelector)][not(Repeat)]">
        <xsl:variable name="portName" select="@tag"/>
        <xsl:variable name="tokenType" select="/Actor/InputPorts/PortDecl[@name=$portName]/Type"/>
        <xsl:variable name="nTokens" select="count(Var)"/>
        <xsl:variable name="seqVar" select="concat('InputSeq$', $portName)"/>
        <xsl:variable name="guardVar" select="concat('Guard$', $portName)"/>


        <!-- Create standard elements (pattern, guard variable, guard) -->

        <xsl:call-template name="createStandardElementsSingle">
            <xsl:with-param name="portName"><xsl:value-of select="$portName"/></xsl:with-param>
            <xsl:with-param name="nTokensExpr">
                <Expr kind="Literal" literal-kind="integer">
                    <xsl:value-of select="$nTokens"/>
                </Expr>
            </xsl:with-param>
        </xsl:call-template>

        <!-- Create element variables -->

        <xsl:for-each select="Var">
            <VarDecl assignable="no" mutable="no" name="{@name}">
                <xsl:copy-of select="$tokenType"/>
                <Expr kind="If">
                    <Expr kind="Var" name="{$guardVar}" old="no"/>
                    <Expr kind="Indexer">
                        <Expr kind="Var" name="{$seqVar}" old="no"/>
                        <Args>
                            <Expr kind="Literal" literal-kind="integer">
                                <xsl:value-of select="position()-1"/>
                            </Expr>
                        </Args>
                    </Expr>
                    <Expr kind="Literal" literal-kind="null"/>
                </Expr>
             </VarDecl>
        </xsl:for-each>

        <!-- Create tail variable -->

        <xsl:if test="SeqVar">
            <VarDecl assignable="no" mutable="no" name="{@name}">
                <Type name="Seq">
                    <TypePars>
                        <xsl:copy-of select="$tokenType"/>
                    </TypePars>
                </Type>
                <Expr kind="If">
                    <Expr kind="Var" name="{$guardVar}" old="no"/>
                    <Expr kind="Application">
                        <Expr kind="Var" name="$tail" old="no"/>
                        <Args>
                            <Expr kind="Literal" literal-kind="integer">
                                <xsl:value-of select="$nTokens"/>
                            </Expr>
                            <Expr kind="Var" name="{$seqVar}" old="no"/>
                        </Args>
                    </Expr>
                    <Expr kind="Literal" literal-kind="null"/>
                </Expr>
             </VarDecl>
        </xsl:if>
    </xsl:template>

    <!--=========================================================================
            input pattern:  P repeat R
    ==========================================================================-->
    <xsl:template match="InputPattern[not(ChannelSelector)][Repeat]">
        <xsl:variable name="portName" select="@tag"/>
        <xsl:variable name="tokenType" select="/Actor/InputPorts/PortDecl[@name=$portName]/Type"/>
        <xsl:variable name="nTokens" select="count(Var)"/>
        <xsl:variable name="seqVar" select="concat('InputSeq$', $portName)"/>
        <xsl:variable name="guardVar" select="concat('Guard$', $portName)"/>
        <xsl:variable name="repeatVar" select="concat('Repeat$', $portName)"/>

        <!-- Create repeat variable -->

        <VarDecl assignable="no" mutable="no" name="{$repeatVar}">
            <Type name="Integer"/>
            <xsl:copy-of select="Repeat/Expr"/>
        </VarDecl>

        <!-- Create standard elements (pattern, guard variable, guard) -->

        <xsl:call-template name="createStandardElementsSingle">
            <xsl:with-param name="portName"><xsl:value-of select="$portName"/></xsl:with-param>
            <xsl:with-param name="nTokensExpr">
                <Expr kind="BinOpSeq">
                    <Operators>
                        <Op name="*"/>
                    </Operators>
                    <Operands>
                        <Expr kind="Literal" literal-kind="integer">
                            <xsl:value-of select="$nTokens"/>
                        </Expr>
                        <Expr kind="Var" name="{$repeatVar}"/>
                    </Operands>
                </Expr>
            </xsl:with-param>
        </xsl:call-template>

        <!-- Create element variables -->

         <xsl:for-each select="Var">
            <VarDecl assignable="no" mutable="no" name="{@name}">
                <xsl:copy-of select="$tokenType"/>
                <Expr kind="If">
                    <Expr kind="Var" name="{$guardVar}" old="no"/>
                    <Expr kind="List">
                        <Expr kind="Indexer">
                            <Expr kind="Var" name="{$seqVar}" old="no"/>
                            <Args>
                                <Expr kind="BinOpSeq">
                                    <Operators>
                                        <Op name="+"/>
                                    </Operators>
                                    <Operands>
                                        <Expr kind="BinOpSeq">
                                            <Operators>
                                                <Op name="*"/>
                                            </Operators>
                                            <Operands>
                                                <Expr kind="Var" name="{$repeatVar}" old="no"/>
                                                <Expr kind="Var" name="$i" old="no"/>
                                            </Operands>
                                        </Expr>
                                        <Expr kind="Literal" literal-kind="integer">
                                            <xsl:value-of select="position()-1"/>
                                        </Expr>
                                    </Operands>
                                </Expr>
                            </Args>
                        </Expr>
                        <Generator>
                            <Type name="Integer"/>
                            <Var name="$i"/>
                            <Expr kind="Application">
                                <Expr kind="Var" name="$Integers" old="no"/>
                                <Args>
                                    <Expr kind="Literal" literal-kind="integer">0</Expr>
                                    <Expr kind="BinOpSeq">
                                        <Operators>
                                            <Op name="-"/>
                                        </Operators>
                                        <Operands>
                                            <Expr kind="Var" name="{$repeatVar}" old="no"/>
                                            <Expr kind="Literal" literal-kind="integer">1</Expr>
                                        </Operands>
                                    </Expr>
                                </Args>
                            </Expr>
                        </Generator>
                    </Expr>
                    <Expr kind="Literal" literal-kind="null"/>
                </Expr>
             </VarDecl>
        </xsl:for-each>

    </xsl:template>

    <!--=========================================================================
            input pattern:  P at C
    ==========================================================================-->
    <xsl:template match="InputPattern[ChannelSelector[@multi='no']][not(Repeat)]">
        <xsl:variable name="portName" select="@tag"/>
        <xsl:variable name="tokenType" select="/Actor/InputPorts/PortDecl[@name=$portName]/Type"/>
        <xsl:variable name="nTokens" select="count(Var)"/>
        <xsl:variable name="seqVar" select="concat('InputSeq$', $portName)"/>
        <xsl:variable name="guardVar" select="concat('Guard$', $portName)"/>
        <xsl:variable name="selectorVar" select="concat('Channel$', $portName)"/>

        <!-- Create selector variable -->

        <VarDecl assignable="no" mutable="no" name="{$selectorVar}">
            <Type name="Integer"/>
            <xsl:copy-of select="ChannelSelector/Expr"/>
        </VarDecl>

        <!-- FIXME Create standard elements (pattern, guard variable, guard) -->

        <xsl:call-template name="createStandardElementsSingle">
            <xsl:with-param name="portName"><xsl:value-of select="$portName"/></xsl:with-param>
            <xsl:with-param name="nTokensExpr">
                <Expr kind="Literal" literal-kind="integer">
                    <xsl:value-of select="$nTokens"/>
                </Expr>
            </xsl:with-param>
            <xsl:with-param name="seqExpr">
                <Expr kind="Literal" literal-kind="integer">
                    <xsl:value-of select="$nTokens"/>
                </Expr>
            </xsl:with-param>
        </xsl:call-template>

        <!-- Create element variables -->

        <xsl:for-each select="Var">
            <VarDecl assignable="no" mutable="no" name="{@name}">
                <xsl:copy-of select="$tokenType"/>
                <Expr kind="If">
                    <Expr kind="Var" name="{$guardVar}" old="no"/>
                    <Expr kind="Indexer">
                        <Expr kind="Var" name="{$seqVar}" old="no"/>
                        <Args>
                            <Expr kind="Literal" literal-kind="integer">
                                <xsl:value-of select="position()-1"/>
                            </Expr>
                        </Args>
                    </Expr>
                    <Expr kind="Literal" literal-kind="null"/>
                </Expr>
             </VarDecl>
        </xsl:for-each>

        <!-- Create tail variable -->

        <xsl:if test="SeqVar">
            <VarDecl assignable="no" mutable="no" name="{@name}">
                <Type name="Seq">
                    <TypePars>
                        <xsl:copy-of select="$tokenType"/>
                    </TypePars>
                </Type>
                <Expr kind="If">
                    <Expr kind="Var" name="{$guardVar}" old="no"/>
                    <Expr kind="Application">
                        <Expr kind="Var" name="$tail" old="no"/>
                        <Args>
                            <Expr kind="Literal" literal-kind="integer">
                                <xsl:value-of select="$nTokens"/>
                            </Expr>
                            <Expr kind="Var" name="{$seqVar}" old="no"/>
                        </Args>
                    </Expr>
                    <Expr kind="Literal" literal-kind="null"/>
                </Expr>
             </VarDecl>
        </xsl:if>
    </xsl:template>


    <xsl:template name="createStandardElementsSingle">
        <xsl:param name="portName"/>
        <xsl:variable name="seqVar"><xsl:value-of select="concat('InputSeq$', $portName)"/></xsl:variable>
        <xsl:param name="seqExpr"><SeqVar name="{$seqVar}"/></xsl:param>
        <xsl:param name="nTokensExpr"/>
        <xsl:variable name="guardVar"><xsl:value-of select="concat('Guard$', $portName)"/></xsl:variable>

        <!-- Create canonical input port pattern -->
        <InputPattern tag="{$portName}">
            <SeqVar name="{$seqVar}"/>
        </InputPattern>

        <!-- Create guard variable -->

        <VarDecl assignable="no" mutable="no" name="{$guardVar}">
            <Type name="Boolean"/>
            <Expr kind="Application">
                <Expr kind="Var" name="$available" old="no"/>
                <Args>
                    <xsl:copy-of select="$nTokensExpr"/>
                    <xsl:copy-of select="$seqExpr"/>
                </Args>
            </Expr>
        </VarDecl>

        <!-- Create guard -->

        <Guard>
            <Expr kind="Var" name="{$guardVar}" old="no"/>
        </Guard>

    </xsl:template>

    <xsl:template match="*">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>
            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
