<!--
    GenerateCoreActor

    This generates the Java actor core from a CAL description.

    author: JWJ
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1"
  xmlns:exsl="http://exslt.org/common"
  xmlns:set="http://exslt.org/sets"
  extension-element-prefixes="exsl set">
    <xsl:output method="text"/>


<!--
    Actor
-->

    <xsl:template match = "Actor">

public class <xsl:value-of select="@name"/>Core {

    private caltrop.runtime.CoreFactory _coreFactory;
    private caltrop.runtime.Action _firableAction = null;
    <xsl:for-each select="./Note[@kind='freeVar']">
    private Object  __<xsl:value-of select="@name"/>;
    </xsl:for-each>

    <!-- Variables (non-assignable, immutable) with actor scope. -->
    <xsl:for-each select="./VarDecl[@assignable='no'][@mutable='no']">
    private Object __<xsl:value-of select="@name"/>; </xsl:for-each>

    <!-- Variables (assignable, immutable) with actor scope. -->
    <xsl:for-each select="./VarDecl[@assignable='yes'][@mutable='no']">
    private caltrop.runtime.Var __<xsl:value-of select="@name"/>; </xsl:for-each>

    <!-- Actor constructor. -->
    public <xsl:value-of select="@name"/>Core(caltrop.runtime.CoreFactory coreFactory) {
        this._coreFactory = coreFactory;
        <xsl:for-each select="Note[@kind='freeVar']">
        __<xsl:value-of select="@name"/> = _coreFactory.getGlobalObject("<xsl:value-of select="@name"/>");</xsl:for-each>
    }

    <xsl:apply-templates select="Action"/>

    <xsl:apply-templates select="*" mode="create-inner-scopes"/>
}
    </xsl:template>


<!--
    Action
-->
    <xsl:template match="Action">
        <xsl:variable name="actionName"><xsl:value-of select="concat('Action$', generate-id())"/></xsl:variable>

    public class <xsl:value-of select="$actionName"/> implements caltrop.runtime.Action {

    <!-- Input sequence variables. -->
    <xsl:for-each select="InputPattern">
        caltrop.runtime.Sequence __<xsl:value-of select="SeqVar/@name"/>;</xsl:for-each>

    <!-- Variables (non-assignable, immutable) with action scope. -->
    <xsl:for-each select="./VarDecl[@assignable='no'][@mutable='no']">
        Object __<xsl:value-of select="@name"/>; </xsl:for-each>

    <!-- Variables (assignable, immutable) with action scope. -->
    <xsl:for-each select="./VarDecl[@assignable='yes'][@mutable='no']">
        caltrop.runtime.Var __<xsl:value-of select="@name"/>; </xsl:for-each>

    <!-- prefire() -->
        public boolean  prefire() throws Exception {
            <xsl:for-each select="InputPattern">
                __<xsl:value-of select="SeqVar/@name"/> = _coreFactory.getInputSequence("<xsl:value-of select="@tag"/>_sequence"); </xsl:for-each>
            <xsl:for-each select="VarDecl">
                <xsl:choose>
                <xsl:when test="@assignable='yes' and @mutable='no'">
                    <xsl:choose>
                    <xsl:when test="Expr">__<xsl:value-of select="@name"/> = _coreFactory.createVar(null, <xsl:call-template name="VarInitValue"><xsl:with-param name="initExpr" select="Expr"/></xsl:call-template>);</xsl:when>
                    <xsl:otherwise>__<xsl:value-of select="@name"/> = _coreFactory.createVar(null, null);</xsl:otherwise>
                    </xsl:choose></xsl:when>
                <xsl:otherwise>
                    __<xsl:value-of select="@name"/> = <xsl:apply-templates select="Expr" mode="value"/>;</xsl:otherwise></xsl:choose>
            </xsl:for-each>

            return (<xsl:for-each select="Guard">
                <xsl:apply-templates select="Expr" mode="value"/><xsl:if test="position()!=last()"> &amp;&amp; </xsl:if></xsl:for-each>);

        }
    <!-- fire() -->

        public void  fire() throws Exception {
            <xsl:apply-templates select="Body/Stmt"/>
        }

        <xsl:apply-templates select="*" mode="create-inner-scopes"/>
    }
    </xsl:template>


    <xsl:template name="VarInitValue">
        <xsl:param name="initExpr"/>

        <xsl:choose>
        <xsl:when test="exsl:node-set($initExpr)"><xsl:apply-templates select="exsl:node-set($initExpr)" mode="value"/></xsl:when>
        <xsl:otherwise>null</xsl:otherwise>
        </xsl:choose>
    </xsl:template>


<!--
    Statements
-->

    <xsl:template match="Stmt[@kind='Assignment']">
        <xsl:choose>
        <xsl:when test="Args">$$$ERROR: Indexed assignment not yet supported.$$$</xsl:when>
        <xsl:otherwise>__<xsl:value-of select="@name"/>.assign(<xsl:apply-templates select="Expr" mode="value"/>);</xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="Stmt[@kind='Exec']">
            ((caltrop.runtime.ProcedureFactory)<xsl:apply-templates select="Expr" mode="value"/>).create().exec(<xsl:apply-templates select="Args"/>);
    </xsl:template>


<!--
    Expressions mode="value"
-->


    <xsl:template match="Expr[@kind='Var']" mode="value">
        <xsl:choose>
            <xsl:when test="Note[@kind='varRef' and @assignable='no' and @mutable='no']">__<xsl:value-of select="@name"/></xsl:when>
            <xsl:otherwise>__<xsl:value-of select="@name"/>.value()</xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="Expr[@kind='Application']" mode="value">
        <xsl:text>((caltrop.runtime.ClosureFactory)</xsl:text>
            <xsl:apply-templates select="Expr" mode="value"/>
        <xsl:text>).create().apply(</xsl:text>
            <xsl:apply-templates select="Args"/>
        <xsl:text>)</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='If']" mode="value">
        <xsl:text>((_coreFactory.booleanValue(</xsl:text>
        <xsl:apply-templates select="Expr[1]" mode="value"/>
        <xsl:text>)) ? </xsl:text><xsl:apply-templates select="Expr[2]" mode="value"/> : <xsl:apply-templates select="Expr[3]" mode="value"/><xsl:text> )</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Indexer']" mode="value">
        <xsl:text>((caltrop.runtime.ClosureFactory)__$$indexer).create().apply(new Object{</xsl:text>
        <xsl:apply-templates select="Expr" mode="value"/><xsl:text>, </xsl:text>
        <xsl:apply-templates select="Args"/><xsl:text>})</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Lambda']" mode="value">
        <xsl:text>(new ClosureFactory$</xsl:text><xsl:value-of select="generate-id()"/><xsl:text>())</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Literal' and @literal-kind='integer']" mode="value">
        <xsl:text>_coreFactory.createInteger(</xsl:text><xsl:value-of select="./text()"/><xsl:text>)</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Literal' and @literal-kind='null']" mode="value">
        <xsl:text>_coreFactory.createNull()</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Literal' and @literal-kind='true']" mode="value">
        <xsl:text>_coreFactory.createBoolean(true)</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Literal' and @literal-kind='false']" mode="value">
        <xsl:text>_coreFactory.createBoolean(false)</xsl:text>
    </xsl:template>

    <xsl:template match="Expr[@kind='Proc']" mode="value">
        <xsl:text>(new ProcedureFactory$</xsl:text><xsl:value-of select="generate-id()"/><xsl:text>())</xsl:text>
    </xsl:template>

    <xsl:template match="Args">
        <xsl:choose>
        <xsl:when test="count(Expr) = 0">
            <xsl:text>null</xsl:text></xsl:when>
        <xsl:when test="count(Expr) = 1">
            <xsl:apply-templates select="Expr" mode="value"/></xsl:when>
        <xsl:otherwise>
            <xsl:text> new Object[] {</xsl:text>
            <xsl:for-each select="Expr">
                <xsl:apply-templates select="." mode="value"/>
                <xsl:if test="position()!=last()">, </xsl:if>
            </xsl:for-each>
            <xsl:text>}</xsl:text></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="Expr">ERROR: Expr template requires mode.</xsl:template>

<!--
    Expressions mode="create-inner-scopes"

    In this mode, the inner classes representing scopes (currently functions and procedures)
    are created, including their nested scopes and so on. They are created along with their
    factory classes.
-->

    <!-- Create Lambda closure class, and its factory. -->
    <xsl:template match="Expr[@kind='Lambda']" mode="create-inner-scopes">
class  Lambda$<xsl:value-of select="generate-id()"/> implements caltrop.runtime.Closure {
        <!-- Parameter variables (immutable, non-assignable). -->
    <xsl:for-each select="NameDecl">
    Object __<xsl:value-of select="@name"/>; </xsl:for-each>
        <!-- Local variables. -->
    <xsl:for-each select="VarDecl">
    caltrop.runtime.Var __<xsl:value-of select="@name"/>; </xsl:for-each>

        <!-- The apply method. -->
    public Object apply(Object arg) {
        <!-- First copy the argument/s. -->
    <xsl:choose>
    <xsl:when test="count(NameDecl)=0"></xsl:when>
    <xsl:when test="count(NameDecl)=1">
        __<xsl:value-of select="NameDecl/@name"/> = arg;</xsl:when>
    <xsl:otherwise>
        Object [] args = (Object [])arg;
        <xsl:for-each select="NameDecl">
        __<xsl:value-of select="@name"/> = args[<xsl:value-of select="position()-1"/>];</xsl:for-each></xsl:otherwise>
    </xsl:choose>
        <!-- Then initialize the local variables. -->
    <xsl:for-each select="VarDecl">
        __<xsl:value-of select="@name"/> = _coreFactory.createVar(null, <xsl:call-template name="VarInitValue"><xsl:with-param name="initExpr" select="Expr"/></xsl:call-template>);</xsl:for-each>
        <!-- Finally, compute the return value. -->
        return <xsl:apply-templates select="Expr" mode="value"/>;
    }

    <xsl:apply-templates select="*" mode="create-inner-scopes"/>
}

class ClosureFactory$<xsl:value-of select="generate-id()"/> implements caltrop.runtime.ClosureFactory {
    public caltrop.runtime.Closure create() {
        return new Lambda$<xsl:value-of select="generate-id()"/>();
    }
}
    </xsl:template>

    <xsl:template match="Expr[@kind='Proc']" mode="create-inner-scopes">
class  Proc$<xsl:value-of select="generate-id()"/> implements caltrop.runtime.Procedure {
    <xsl:for-each select="NameDecl">
    Object __<xsl:value-of select="@name"/>; </xsl:for-each>
    <xsl:for-each select="VarDecl">
    caltrop.runtime.Var __<xsl:value-of select="@name"/>; </xsl:for-each>

    public void exec(Object arg) {
    <xsl:choose>
    <xsl:when test="count(NameDecl)=0"></xsl:when>
    <xsl:when test="count(NameDecl)=1">
        __<xsl:value-of select="NameDecl/@name"/> = arg;</xsl:when>
    <xsl:otherwise>
        Object [] args = (Object [])arg;
        <xsl:for-each select="NameDecl">
        __<xsl:value-of select="@name"/> = args[<xsl:value-of select="position()-1"/>];</xsl:for-each></xsl:otherwise>
    </xsl:choose>

    <xsl:for-each select="VarDecl">
        __<xsl:value-of select="@name"/> = _coreFactory.createVar(null, <xsl:call-template name="VarInitValue"><xsl:with-param name="initExpr" select="Expr"/></xsl:call-template>);</xsl:for-each>

    <xsl:apply-templates select="Stmt"/>
    }

    <xsl:apply-templates select="*" mode="create-inner-scopes"/>
}

class ProcedureFactory$<xsl:value-of select="generate-id()"/> implements caltrop.runtime.ProcedureFactory {
    public caltrop.runtime.Procedure create() {
        return new Proc$<xsl:value-of select="generate-id()"/>();
    }
}

    </xsl:template>

    <xsl:template match="Action" mode="create-inner-scopes"/>

    <xsl:template match="*" mode="create-inner-scopes">
        <xsl:apply-templates select="*" mode="create-inner-scopes"/>
    </xsl:template>

</xsl:stylesheet>