<!--
    GeneratePtActor

    This generates a Ptolemy adaptor for a specific CAL actor.

    author: JWJ
-->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1">
    <xsl:output method="text"/>

    <xsl:template match = "Actor">
public class <xsl:value-of select="@name"/> extends ptolemy.actor.TypedAtomicActor {
    public <xsl:value-of select="@name"/>(ptolemy.kernel.CompositeEntity container, String name)
    throws ptolemy.kernel.util.NameDuplicationException, ptolemy.kernel.util.IllegalActionException {
		super(container, name);
		_varChangeListener = new caltrop.runtime.ptolemy.PtVarChangeListener();
		_coreFactory = new caltrop.runtime.ptolemy.TheFactory(_varChangeListener);
		try {
		_actorCore = new <xsl:value-of select="@name"/>Core(_coreFactory);
		} catch(Exception e) { throw new RuntimeException("Exception in core ctor: " + e.getMessage()); }
<xsl:for-each select="InputPorts/PortDecl">
    <xsl:variable name = "type">
        <xsl:call-template name="PtType">
            <xsl:with-param name="type"><xsl:value-of select="Type"/></xsl:with-param>
        </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="@name"/> = new ptolemy.actor.TypedIOPort(this, "<xsl:value-of select="@name"/>", true, false);
    <xsl:value-of select="@name"/>.setTypeEquals(<xsl:value-of select="$type"/>);
</xsl:for-each>
    <xsl:variable name = "type">
        <xsl:call-template name="PtType">
            <xsl:with-param name="type"><xsl:value-of select="Type"/></xsl:with-param>
        </xsl:call-template>
    </xsl:variable>
        <xsl:for-each select="OutputPorts/PortDecl">
            <xsl:value-of select="@name"/> = ptolemy.actor.new TypedIOPort(this, "<xsl:value-of select="@name"/>", false, true);
</xsl:for-each>

	public boolean prefire() throws ptolemy.kernel.util.IllegalActionException
	{
    <xsl:for-each select="InputPorts/PortDecl">
        <xsl:value-of select="@name"/>_channel.clear();
    </xsl:for-each>
    }
	private RuntimeException createForwardException(String location, Exception e)
	{
		java.io.StringWriter sw = new java.io.StringWriter();
		e.printStackTrace(new java.io.PrintWriter(sw));
		return new RuntimeException("Error in '" + location + "': " + e.getMessage() + " -- " + sw);
	}

	public boolean postfire() throws ptolemy.kernel.util.IllegalActionException
	{
		_varChangeListener.commitAll();
		return true;
	}

	public void fire()
	{
		_varChangeListener.rollBackAll();
		try {
		if (_actorCore.prefire()) {
		_actorCore.fire();
		}
		} catch (Exception e) {
		throw createForwardException("ActorCore.prefire", e);
		}
		return;
	}

	private caltrop.runtime.ptolemy.PtVarChangeListener _varChangeListener;
	private caltrop.runtime.ptolemy.TheFactory _coreFactory;
    private <xsl:value-of select="@name"/>Core _actorCore;
<xsl:for-each select="InputPorts/PortDecl">
    public ptolemy.actor.TypedIOPort <xsl:value-of select="@name"/>;
    public caltrop.runtime.ptolemy.InputChannel <xsl:value-of select="@name"/>_channel;
</xsl:for-each><xsl:for-each select="OutputPorts/PortDecl">
    public ptolemy.actor.TypedIOPort <xsl:value-of select="@name"/>;
</xsl:for-each>

	public void initialize() throws ptolemy.kernel.util.IllegalActionException
	{
		_coreFactory.clearAll();
<xsl:for-each select="InputPorts/PortDecl">
		<xsl:value-of select="@name"/>_channel =
            new caltrop.runtime.ptolemy.InputChannel(<xsl:value-of select="@name"/>);
		_coreFactory.addPtObject("<xsl:value-of select="@name"/>_sequence"
            , <xsl:value-of select="@name"/>_channel);
		</xsl:for-each><xsl:for-each select="OutputPorts/PortDecl">
        _coreFactory.addPtObject("<xsl:value-of select="@name"/>"
            , <xsl:value-of select="@name"/>);
</xsl:for-each>
		try {
		_actorCore.initialize();
		} catch(Exception e) { throw createForwardException("ActorCore.initialize", e); }
	}
}


    </xsl:template>

    <!-- FIXME -->
    <xsl:template name="PtType">
        <xsl:param name="type"/>

        ptolemy.data.type.BaseType.DOUBLE
    </xsl:template>

</xsl:stylesheet>
