<!--
    ReplaceOld

    Replaces in each action all old-references to variables defined inside the entire action scope with
    references to a temporary variable $old$v (for a variable v). It also introduces the definition
    if $old$v into the action scope.

    author: YZ, JWJ
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:calext="caltrop.dom.xalan.Extension"
  version="1.1">
  <xsl:output method="xml"/>


  <xsl:template match="Action">
    <xsl:variable name="this" select="."/>
    <xsl:variable name="oldRefs" select=".//Expr[@kind='Var'][@old='yes'][Note[@kind='varRef']/@scope-id=$this/ancestor-or-self::*/@id]"/>

        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:for-each select="$oldRefs">
              <xsl:variable name="myName" select="@name"/>
              <xsl:if test="not(preceding-sibling::*[@name=$myName])">
                <VarDecl name="$old${@name}" mutable="no" assignable="no">
                  <xsl:copy-of select="Note[@kind='varRef']/Type"/>
                  <Expr kind="Var" name="{@name}" old="no"/>
                </VarDecl>
              </xsl:if>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
  </xsl:template>

  <xsl:template match="Expr[@kind='Var'][@old='yes']">
     <Expr kind="Var" name="$old${@name}" old="no"/>
  </xsl:template>


    <xsl:template match="*">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>


</xsl:stylesheet>


