<!--
    TagPorts

    Adds a name tag to each input/output pattern that does not have one.
    It does so by positional matching with the actor's port declarations.

    author: JWJ
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:calext="caltrop.dom.xalan.Extension"
  version="1.0">
  <xsl:output method="xml"/>


    <xsl:template match="InputPattern">
        <xsl:variable name="pos" select="count(preceding-sibling::InputPattern)+1"/>
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>


            <xsl:if test="not(@tag)">
                <xsl:attribute name="tag"><xsl:value-of select="ancestor::Actor[1]/InputPorts/PortDecl[$pos]/@name"/></xsl:attribute>
            </xsl:if>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="OutputPattern">
        <xsl:variable name="pos" select="count(preceding-sibling::OutputPattern)+1"/>
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>


            <xsl:if test="not(@tag)">
                <xsl:attribute name="tag"><xsl:value-of select="ancestor::Actor[1]/OutputPorts/PortDecl[$pos]/@name"/></xsl:attribute>
            </xsl:if>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>


    <xsl:template match="*">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>


</xsl:stylesheet>


