<!--
    VariableSorter

    Sorts variable declarations according to their dependencies.

    assumes: DependencyAnnotator

    author: JWJ
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1"
  xmlns:exsl="http://exslt.org/common"
  xmlns:set="http://exslt.org/sets"
  extension-element-prefixes="exsl set">

  <xsl:output method="xml"/>


    <xsl:template match="Actor | Action | Expr[@kind='Proc'] | Expr[@kind='Lambda'] | Stmt[@kind='Block']">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>

            <!--
                Sorts declarations. The basic idea is that the number of non-lazy dependencies
                indicates the precedence. If a variable v1 depends on v2, then that number is
                strictly higher for v1 than v2, as v1 depends on everything v2 depends on, plus v2.
            -->
            <xsl:apply-templates select="VarDecl" mode="copy">
                <xsl:sort select="count(Note[@kind='dependency'][@lazy='no'])"/>
            </xsl:apply-templates>

        </xsl:copy>
    </xsl:template>

    <!-- Swallow VarDecl elements in the first pass. -->
    <xsl:template match="VarDecl"/>

    <!-- Copy VarDecl elements in the second pass. -->
    <xsl:template match="VarDecl" mode="copy">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>


    <xsl:template match="*">
        <xsl:copy>
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
            </xsl:for-each>

            <xsl:apply-templates select="node() | text()"/>
        </xsl:copy>
    </xsl:template>


</xsl:stylesheet>
